<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2005 THAMEY Support (support@thamey.de)
*  All rights reserved
*
*  This script is not part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Plugin 'TM ASIN Search' for the 'tm_asinsearch' extension.
 *
 * @author	THAMEY Support <support@thamey.de>
 * @author	Ronald Steiner for patch "P1"
 */


require_once(PATH_tslib . 'class.tslib_pibase.php');

class tx_tmasinsearch_pi1 extends tslib_pibase {
	var $prefixId = 'tx_tmasinsearch_pi1';			// Same as class name
	var $scriptRelPath = 'pi1/class.tx_tmasinsearch_pi1.php';		// Path to this script relative to the extension dir.
	var $extKey = 'tm_asinsearch';				// The extension key.
	var $pi_checkCHash = TRUE;

	var $debug = 0;

	var $accessKey;
	var $associateID;
	var $wsdlFile;
	var $locale;
	var $resultType;
	var $templateFile;
	var $templateVersion;

	function main($content, $conf) {
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->pi_initPIflexForm();

		/*** read settings from page template ***/

		/* get setting for debug mode; default is OFF */
		$this->debug = $this->conf['debug'];

		/* get settings for locale and set standard SOAP server; default is DE */
		$this->locale = strtoupper($this->conf['locale']);
		switch($this->locale) {
			case 'CA':
				$this->wsdlFile = 'ca';
				break;
			case 'US':
				$this->wsdlFile = 'com';
				break;
			case 'FR':
				$this->wsdlFile = 'fr';
				break;
			case 'JP':
				$this->wsdlFile = 'jp';
				break;
			case 'UK':
				$this->wsdlFile = 'co.uk';
				break;
			default:
				$this->locale = 'DE';
				$this->wsdlFile = 'de';
		}
		$this->wsdlFile = 'http://ecs.amazonaws.' . $this->wsdlFile . '/onca/xml?Service=AWSECommerceService';

		/* override standard SOAP server, if specified in typoscript */
		if(!empty($this->conf['wsdlFile'])) {
			$this->wsdlFile = $this->conf['wsdlFile'];
		}

		/* get access key from configuration; this does not default to anything */
		$this->accessKey = $this->conf['access_key'];
		if(empty($this->accessKey)) {				// if this key is not configured
			$content .= $this->pi_getLL('accessKeyError');	// show error message
			return $this->pi_wrapInBaseClass($content);		// and exit
		}

		/* get associate ID from configuration; this defaults to ID from "Haustierratgeber.de", but can be erased in TS */
		$this->associateID = $this->conf['associate_id'];

		/* get template file from configuration; defaults to standard template file */
		$this->templateFile = $this->conf['templateFile'];
		if(empty($this->templateFile)) {
			$this->templateFile = 'EXT:tm_asinsearch/sample_templates/sample.tmpl';
		}

		/* get result type from configuration; defaults to result type "Medium" */
		$this->resultType = $this->conf['resultType'];
		if(empty($this->resultType))
			$this->resultType = 'Medium';

		/* get ASIN from content element if available */
		$asin = trim($this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'inputAsin', 'sDEF'));
		if(empty($asin)) {					// if ASIN from CE is not available
			$asin = $this->conf['asin'];			// try to get from configuration
		}
		if(empty($asin)) {					// if ASIN still not available
			$content .= $this->pi_getLL('missingASIN');		// output error message
			return $this->pi_wrapInBaseClass($content);		// and exit
		}


		/* if in debug mode 2 (verbose), output the effective settings */
		if($this->debug == 2) {
			$arrDebug = array();
			$arrDebug['accessKey'] = $this->accessKey;
			$arrDebug['associateID'] = $this->associateID;
			$arrDebug['wsdlFile'] = $this->wsdlFile;
			$arrDebug['locale'] = $this->locale;
			$arrDebug['resultType'] = $this->resultType;
			$arrDebug['templateFile'] = $this->templateFile;
			$arrDebug['asin'] = $asin;

			t3lib_div::debug("Configuration settings");
			t3lib_div::debug($arrDebug);
		}


		/*** do the query ***/
		$queryResult = $this->queryAWS_REST($asin);

		/* if in any debug mode, output the query result */
		if($this->debug) {
			t3lib_div::debug("Query result");
			t3lib_div::debug(empty($queryResult) ? 'Error quering amazon' : $queryResult);
		}

		/* retrieve template only, if amazon query was successful */
		if(!empty($queryResult)) {
			$templateCode = $this->getTemplate();
			
			/* if in debug mode 2 (verbose), output the read template */
			if($this->debug == 2) {
				t3lib_div::debug("Template retrieval result");
				t3lib_div::debug(empty($queryResult) ? 'Error retrieving template' : $templateCode);
			}

			/* fill markers only, if amazon query and template retrieval was successful */
			if(!empty($templateCode)) {
				if($this->templateVersion == 2) {
					$content .= $this->fillMarkers_V2($templateCode, $queryResult);
				} else {
					$content .= $this->fillMarkers($templateCode, $queryResult);
				}
			}
		}
		
		$content = $this->cleanTemplate($content);

		return $this->pi_wrapInBaseClass($content);
	}


	/* Query amazon webservice for requested ASIN and retrieve information using REST */
	function queryAWS_REST($asin) {
		/* check, if all required parameters are set */
		if(empty($this->accessKey) || empty($this->wsdlFile) || empty($asin)) {
			return false;
		}

		$request = $this->wsdlFile;
		$request .= '&AWSAccessKeyId=' . $this->accessKey;
		
		if(!empty($this->associateID)) {
			$request .= '&AssociateTag=' . $this->associateID;
		}

		$request .= '&Operation=ItemLookup';
		$request .= '&ItemId=' . $asin;

		if(!empty($this->resultType)) {
			$request .= '&ResponseGroup=' . $this->resultType;
		}

		$responseXML = t3lib_div::getURL($request, 0);
		$result = t3lib_div::xml2array($responseXML);
		
		/* patch "P1" by Ronald Steiner */
		if(is_array($result) && ($result['Items']['Request']['IsValid'] == 'True')) {
			return $result['Items']['Item'];
		} else {
			return false;
		}
	}


	/* Load template, detect template version and separate subpart markers */
	function getTemplate() {
		/* read in template file */
		$this->templateVersion = 0;
		$templateCode = $this->cObj->fileResource($this->templateFile);

		/* check, if template is version 2. Version 2 takes preceedence. */
		$amazonitem = $this->cObj->getSubpart($templateCode, '###AMAZONITEM_V2###');
		if(!empty($amazonitem)) {
			$this->templateVersion = 2;
		} else {
			$amazonitem = $this->cObj->getSubpart($templateCode, '###AMAZONITEM###');
			if(!empty($amazonitem)) {
				$this->templateVersion = 1;
			}
		}
		
		if(!$this->templateVersion || ($this->templateVersion == 0)) {
			return '';
		}
		
		return $amazonitem;
	}
	
	function parseTemplate() {
	}
	
	function cleanTemplate($templateCode) {
		while (strpos($templateCode, '###') > 0) {
			$templateArray = explode('###', $templateCode, 3);
			unset($templateArray[1]);
			$templateCode = implode('', $templateArray);
		}
		
		return $templateCode;
	}

	function fillMarkersRecursive($templateCode, $queryInfos) {
		$nextTemplateCode = $templateCode;

		foreach($queryInfos as $key => $value) {
			if(is_array($value)) {
				$nextPart = $this->cObj->getSubpart($templateCode, '###' . strtoupper($key) . '###');
				if(!empty($nextPart)) {
					$lastResult = $this->fillMarkersRecursive($nextPart, $value);
					$nextTemplateCode = $this->cObj->substituteSubpart($nextTemplateCode, '###' . strtoupper($key) . '###', $lastResult, 0);
				}
			}
		}

		foreach($queryInfos as $key => $value) {
			if(!is_array($value)) {
				$nextTemplateCode = $this->cObj->substituteMarker($nextTemplateCode, '###' . strtoupper($key) . '###', $value);
			}
		}

		return $nextTemplateCode;
	}
	
	function fillMarkers($templateCode, $queryInfos) {
		$result = $this->fillMarkersRecursive($templateCode, $queryInfos);
	
//		$arrMarker['###CUSTOMIMAGE###'] = $this->getImage($queryInfos['LargeImage']['URL']);
		
		return $result;
	}
	
	function fillMarkersRecursive_V2($templateCode, $queryInfos, $prefix) {
		$nextTemplateCode = $templateCode;
		
		foreach($queryInfos as $key => $value) {
			if(!is_array($value)) {
				$nextTemplateCode = $this->cObj->substituteMarker($nextTemplateCode, '###' . strtoupper($prefix . (empty($prefix) ? '' : '_') . $key) . '###', $value);
			} else {
				$nextTemplateCode = $this->fillMarkersRecursive_V2($nextTemplateCode, $value, $prefix . (empty($prefix) ? '' : '_') . $key);
			}
		}

		return $nextTemplateCode;
	}
	
	function fillMarkers_V2($templateCode, $queryInfos) {
		$result = $this->fillMarkersRecursive_V2($templateCode, $queryInfos, '');

		return $result;
	}

	function getImage($imgParam) {
		$imgName = explode('/', $imgParam);

		if (count($imgName) > 0)
			$imgFile = "fileadmin/user_upload/" . $imgName[count($imgName) - 1];
		
		if(!is_file($imgFile)) {
			$imgBinary = file_get_contents($imgParam);
			$fp = fopen('./' . $imgFile, 'wb');
			fputs($fp, $imgBinary);
			fclose($fp);
		}

		$imgTS = $this->conf['image.'];
		
		if(!empty($imgFile))
			$imgTS['file'] = $imgFile;

		$imgCode = $this->cObj->IMAGE($imgTS);

		/* if debug mode 2 (verbose), output the image field */
		if($this->debug == 2) {
			t3lib_div::debug($imgTS);
		}
		
		return $imgCode;
	}
}

if(defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/tm_asinsearch/pi1/class.tx_tmasinsearch_pi1.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/tm_asinsearch/pi1/class.tx_tmasinsearch_pi1.php']);
}

?>
